@@ -53,8 +53,14 @@ class Stub(object):
         self._stubs = OperationList()
         self._setting_up = False
         self._new_attr_hooks = self._new_attr_hooks[:]
+        self._deactivate = False
         self.__class__.__setattr__ = self.__setattr__hook
 
+    def _activate_next(self):
+        self.__enter__()
+        self._deactivate = True
+        return self
+
     def __enter__(self):
         self._setting_up = True
         return self
@@ -79,7 +85,7 @@ class Stub(object):
         actual_retval = self._perform_invocation(invocation)
 
         retval = stubbed_retval if stubbed_retval is not None else actual_retval
-        invocation._context.retval = retval
+        invocation.context.retval = retval
         return retval
 
     def _prepare_invocation(self, invocation):
@@ -125,7 +131,7 @@ class Spy(Stub, SpyBase):
 
     def _get_invocations_to(self, name):
         return [i for i in self._recorded
-                if self._proxy.same_method(name, i._name)]
+                if self._proxy.same_method(name, i.name)]
 
 
 class ProxySpy(Spy):
