@@ -20,14 +20,23 @@
 
 
 import sys
-from unittest import TestCase
 import itertools
-import thread
 import threading
-import io
-import copy
+import thread
 
-from hamcrest import is_not, all_of, contains_string, has_length
+if sys.version_info >= (2, 7):
+    from unittest import TestCase
+else:
+    from unittest2 import TestCase
+
+if sys.version_info >= (3, 0):
+    unicode = str
+    from io import StringIO
+else:
+    from io import BytesIO as StringIO
+
+
+from hamcrest import is_not, all_of, contains_string, has_length, has_entry, starts_with
 from hamcrest.library.text.stringcontainsinorder import *
 from hamcrest.library.object.hasproperty import *
 from hamcrest.library.number.ordering_comparison import *
@@ -149,6 +158,8 @@ class StubTests(TestCase):
             expected = "hello() takes exactly 1 argument (2 given)"
             if sys.version_info >= (3,):
                 expected = "hello() takes exactly 1 positional argument (2 given)"
+            if sys.version_info >= (3, 3):
+                expected = "hello() takes 1 positional argument but 2 were given"
             assert_that(str(e), contains_string(expected))
 
     # bitbucket issue #6
@@ -163,7 +174,7 @@ class StubTests(TestCase):
         assert_that(self.stub.kwarg_method(key_param=6), is_(6000))
         assert_that(self.stub.kwarg_method(key_param=6), is_(6000))
 
-    # FIXME: new on 1.7
+    # new on 1.7
     def test_keyworked_or_positional_are_equivalent(self):
         with self.stub:
             self.stub.kwarg_method(1).returns(1000)
@@ -206,7 +217,7 @@ class AccessingActualAttributes(TestCase):
         stub = Stub(Collaborator())
         assert_that(stub.class_attr, is_("OK"))
 
-    # New in version 1.6.5
+    # New in 1.6.5
     def test_proxyspy_read_instance_attribute(self):
         stub = Stub(Collaborator())
         assert_that(stub.instance_attr, is_(300))
@@ -442,18 +453,24 @@ class BuiltinSpyTests(TestCase):
             expected = "'list' object has no attribute 'wrong'"
             assert_that(str(e), contains_string(expected))
 
+    def test_dict_builtin_method(self):
+        spy = Spy(dict)
+        spy.__setitem__(3, 5)
+        assert_that(spy.__setitem__, called().with_args(3, 5))
+
 
 class ProxySpyTests(TestCase):
     def test_must_give_argument(self):
-        self.failUnlessRaises(TypeError, ProxySpy)
+        with self.assertRaises(TypeError):
+            ProxySpy()
 
     def test_given_argument_can_not_be_oldstyle_class(self):
-        self.failUnlessRaises(TypeError,
-                              ProxySpy, Collaborator)
+        with self.assertRaises(TypeError):
+            ProxySpy(Collaborator)
 
     def test_given_argument_can_not_be_newstyle_class(self):
-        self.failUnlessRaises(TypeError,
-                              ProxySpy, ObjCollaborator)
+        with self.assertRaises(TypeError):
+            ProxySpy(ObjCollaborator)
 
     def test_propagate_stubbed_calls_to_collaborator(self):
         class Foo:
@@ -546,9 +563,8 @@ class MockOrderTests(TestCase):
         self.mock.bar()
         self.mock.foo()
 
-        self.failUnlessRaises(
-            AssertionError,
-            assert_that, self.mock, verify())
+        with self.assertRaises(AssertionError):
+            assert_that(self.mock, verify())
 
     def test_method_name_order_does_not_matter_with_any_order(self):
         with self.mock:
@@ -699,6 +715,8 @@ class ApiMismatchTest(TestCase):
             expected = "Collaborator.hello() takes exactly 1 argument (2 given)"
             if sys.version_info >= (3,):
                 expected = "Collaborator.hello() takes exactly 1 positional argument (2 given)"
+            if sys.version_info >= (3, 3):
+                expected = "hello() takes 1 positional argument but 2 were given"
             assert_that(str(e), contains_string(expected))
 
     def test_fail_wrong_kargs(self):
@@ -847,7 +865,7 @@ class MatcherTests(TestCase):
 
         assert_that(stub.method("awesome"), is_(1000))
 
-    # doc
+    # doc FIXME: assure this is tested with doctests and remove
     def test_times_arg_may_be_matcher(self):
         self.spy.foo()
         self.spy.foo(1)
@@ -869,7 +887,7 @@ class MatcherTests(TestCase):
         assert_that(self.spy.foo, called().with_args(1).times(greater_than(1)))  # > 1
         assert_that(self.spy.foo, called().with_args(1).times(less_than(5)))     # < 5
 
-    # doc
+    # doc FIXME: assure this is tested with doctests and remove
     def test_called_args(self):
         self.spy.m1()
         self.spy.m2(None)
@@ -896,6 +914,18 @@ class MatcherTests(TestCase):
     def test_assert_that_requires_a_matcher(self):
         self.assertRaises(MatcherRequiredError, assert_that, self.spy.m1, True)
 
+    # from pydoubles docs
+    def test_has_entry_matcher(self):
+        with Spy() as spy:
+            spy.one_arg_method(has_entry(is_('two'), 2)).returns(1000)
+
+        assert_that(spy.one_arg_method({'one': 1, 'two': 2}), is_(1000))
+
+    def test_all_of_matcher(self):
+        with Spy() as spy:
+            spy.one_arg_method(all_of(starts_with('h'), instance_of(str))).returns(1000)
+        assert_that(spy.one_arg_method('hello'), is_(1000))
+
 
 class StubObserverTests(TestCase):
     def setUp(self):
@@ -932,7 +962,7 @@ class StubDelegateTests(TestCase):
 
     def test_delegate_to_list(self):
         with self.stub:
-            self.stub.foo().delegates(range(3))
+            self.stub.foo().delegates(list(range(3)))
 
         self.assert_012(self.stub.foo)
 
@@ -985,7 +1015,7 @@ class MockDelegateTest(TestCase):
 
     def test_delegate_to_list_is_only_an_expectation(self):
         with self.mock:
-            self.mock.foo().delegates(range(3))
+            self.mock.foo().delegates(list(range(3)))
 
         self.mock.foo()
         assert_that(self.mock, verify())
@@ -1078,9 +1108,9 @@ class PropertyTests(TestCase):
 
     def test_spy_get_property_fail(self):
         spy = Spy(ObjCollaborator)
-        self.failUnlessRaises(
-            AssertionError,
-            assert_that, spy, property_got('prop'))
+
+        with self.assertRaises(AssertionError):
+            assert_that(spy, property_got('prop'))
 
     def test_spy_set_property_using_class(self):
         spy = Spy(ObjCollaborator)
@@ -1098,9 +1128,9 @@ class PropertyTests(TestCase):
 
     def test_spy_set_property_fail(self):
         spy = Spy(ObjCollaborator)
-        self.failUnlessRaises(
-            AssertionError,
-            assert_that, spy, property_set('prop'))
+
+        with self.assertRaises(AssertionError):
+            assert_that(spy, property_set('prop'))
 
     def test_spy_set_property_to(self):
         spy = Spy(ObjCollaborator)
@@ -1151,18 +1181,18 @@ class PropertyTests(TestCase):
                     property_set('prop').to(greater_than(1)).
                     times(less_than(3)))
 
-    def test_proxy_spy_get_actual_property(self):
+    def test_proxyspy_get_actual_property(self):
         collaborator = ObjCollaborator()
         sut = ProxySpy(collaborator)
         assert_that(sut.prop, is_(1))
 
-    def test_proxy_spy_get_stubbed_property(self):
+    def test_proxyspy_get_stubbed_property(self):
         collaborator = ObjCollaborator()
         with ProxySpy(collaborator) as sut:
             sut.prop = 2
         assert_that(sut.prop, is_(2))
 
-    def test_proxy_spy_set_property(self):
+    def test_proxyspy_set_property(self):
         collaborator = ObjCollaborator()
         sut = ProxySpy(collaborator)
         sut.prop = 20
@@ -1170,6 +1200,82 @@ class PropertyTests(TestCase):
         assert_that(collaborator.prop, is_(20))
 
 
+# FIXME: new on 1.8
+class PropertyMockTests(TestCase):
+    def test_mock_get(self):
+        with Mock(ObjCollaborator) as mock:
+            mock.prop
+
+        mock.prop
+
+        assert_that(mock, verify())
+
+    def test_mock_get_but_never_got(self):
+        with Mock(ObjCollaborator) as mock:
+            mock.prop
+
+        with self.assertRaises(AssertionError):
+            assert_that(mock, verify())
+
+    def test_mock_get_too_many_times(self):
+        with Mock(ObjCollaborator) as mock:
+            mock.prop
+
+        mock.prop
+        mock.prop
+
+        with self.assertRaises(AssertionError):
+            assert_that(mock, verify())
+
+    def test_mock_set(self):
+        with Mock(ObjCollaborator) as mock:
+            mock.prop = 5
+
+        mock.prop = 5
+
+        assert_that(mock, verify())
+
+    def test_mock_set_but_never_set(self):
+        with Mock(ObjCollaborator) as mock:
+            mock.prop = 5
+
+        with self.assertRaises(AssertionError):
+            assert_that(mock, verify())
+
+    def test_mock_set_too_many_times(self):
+        with Mock(ObjCollaborator) as mock:
+            mock.prop = 5
+
+        mock.prop = 5
+        mock.prop = 5
+
+        with self.assertRaises(AssertionError):
+            assert_that(mock, verify())
+
+    def test_mock_set_wrong_value(self):
+        with Mock(ObjCollaborator) as mock:
+            mock.prop = 5
+
+        with self.assertRaises(AssertionError):
+            mock.prop = 8
+
+    def test_mock_set_anything(self):
+        with Mock(ObjCollaborator) as mock:
+            mock.prop = anything()
+
+        mock.prop = 5
+
+        assert_that(mock, verify())
+
+    def test_mock_set_matcher(self):
+        with Mock(ObjCollaborator) as mock:
+            mock.prop = all_of(greater_than(8), less_than(12))
+
+        mock.prop = 10
+
+        assert_that(mock, verify())
+
+
 class AsyncTests(TestCase):
     class SUT(object):
         def __init__(self, collaborator):
@@ -1231,7 +1337,7 @@ class AsyncTests(TestCase):
         assert_that(spy.write, called().async(timeout=1))
 
 
-# FIXME: new on 1.7
+# new on 1.7
 class with_some_args_matcher_tests(TestCase):
     def test_one_arg(self):
         spy = Spy(Collaborator)
@@ -1256,7 +1362,7 @@ class with_some_args_matcher_tests(TestCase):
             assert_that(spy.foo, called().with_some_args())
 
 
-# FIXME: new on 1.7
+# new on 1.7
 class Stub_default_behavior_tests(TestCase):
     def test_set_return_globally(self):
         StubClone = Stub._clone_class()
@@ -1296,7 +1402,7 @@ class Stub_default_behavior_tests(TestCase):
         assert_that(stub.hello(), is_(1000))
 
 
-# FIXME: new on 1.7
+# new on 1.7
 class Spy_default_behavior_tests(TestCase):
     def test_set_return_globally(self):
         SpyClone = Spy._clone_class()
@@ -1317,7 +1423,7 @@ class Spy_default_behavior_tests(TestCase):
         assert_that(spy.unknown, called().with_args(7))
 
 
-# FIXME: new on 1.7
+# new on 1.7
 class ProxySpy_default_behavior_tests(TestCase):
     def test_this_change_proxyspy_default_behavior(self):
         spy = ProxySpy(Collaborator())
@@ -1373,7 +1479,71 @@ class orphan_methods_tests(TestCase):
 #        assert_that(spy.method, called().times(3))
 
 
-# FIXME: new on 1.7.2
+class when_tests(TestCase):
+    def test_stub_when(self):
+        stub = Stub()
+        when(stub).add(2, 2).returns(5)
+        when(stub).add(2, 4).returns(8)
+        when(stub).sub(3, ANY_ARG).returns(100)
+        when(stub).foo(ANY_ARG).returns_input()
+        when(stub).foo(anything(), 7).returns(300)
+
+        assert_that(stub.add(2, 2), is_(5))
+        assert_that(stub.add(2, 4), is_(8))
+        assert_that(stub.sub(3, 600), is_(100))
+        assert_that(stub.foo((8, 2)), is_((8, 2)))
+        assert_that(stub.foo(3, 7), is_(300))
+
+        assert_that(stub.add(), is_(None))
+
+    def test_stub_when_with_args(self):
+        stub = Stub()
+        when(stub).some_calculation(5).returns(10)
+        when(stub).some_calculation(10).returns(20)
+
+        assert_that(stub.some_calculation(5), is_(10))
+
+    def test_spy_when(self):
+        spy = Spy()
+        when(spy).add(2, 2).returns(5)
+        when(spy).add(2, 4).returns(8)
+
+        assert_that(spy.add(2, 2), is_(5))
+        assert_that(spy.add(2, 4), is_(8))
+        assert_that(spy.add(), is_(None))
+
+    # from pydoubles docs
+    def test_has_entry_matcher(self):
+        spy = Spy()
+        when(spy).one_arg_method(has_entry(is_('two'), 2)).returns(1000)
+        assert_that(spy.one_arg_method({'one': 1, 'two': 2}), is_(1000))
+
+    def test_all_of_matcher(self):
+        spy = Spy()
+        when(spy).one_arg_method(all_of(starts_with('h'), instance_of(str))).returns(1000)
+        assert_that(spy.one_arg_method('hello'), is_(1000))
+
+
+class expect_call_tests(TestCase):
+    def test_expect_call(self):
+        mock = Mock()
+        expect_call(mock).add(2, 4)
+        expect_call(mock).add(2, 2).returns(5)
+
+        mock.add(2, 4)
+        assert_that(mock.add(2, 2), is_(5))
+
+        assert_that(mock, verify())
+
+    def test_except_call_with_stub_or_spy_forbidden(self):
+        with self.assertRaises(WrongApiUsage):
+            expect_call(Stub()).foo()
+
+        with self.assertRaises(WrongApiUsage):
+            expect_call(Spy()).foo()
+
+
+# new on 1.7.2
 class VarArgsTest(TestCase):
     def test_stub_args(self):
         stub = Stub(Collaborator)
@@ -1426,10 +1596,10 @@ class VarArgsTest(TestCase):
                         contains_string('with_some_args() can not be applied to method Collaborator.varargs(self, *args, **kargs)'))
 
 
-# FIXME: new on 1.7.2
+# new on 1.7.2
 class TracerTests(TestCase):
     def setUp(self):
-        self.out = io.BytesIO()
+        self.out = StringIO()
         self.tracer = Tracer(self.out.write)
 
     def test_trace_single_method(self):
